# THIS CLASS SHOULD NOT EXISTS
# TODO MOVE THIS TO HUBCLIENT REMOTE_LIB IMPLEMENTATION
import os
import py7zr
import shutil
from typing import Dict, Optional
from functools import cached_property
from cli.constants import *
from cli.utils.loader import Loader
from cli.utils.api_requests import *
from cli.component.spec import Spec, PrivacyPolicy
from splight_lib import logging

logger = logging.getLogger()


class UserHandler:

    def __init__(self, context):
        self.context = context
        self.access_id = self.context.workspace.settings.SPLIGHT_ACCESS_ID
        self.secret_key = self.context.workspace.settings.SPLIGHT_SECRET_KEY
        self.host = self.context.workspace.settings.SPLIGHT_PLATFORM_API_HOST

    @property
    def authorization_header(self):
        return {
            'Authorization': f"Splight {self.access_id} {self.secret_key}"
        }

    @cached_property
    def user_namespace(self):
        org_id = DEFAULT_NAMESPACE
        try:
            headers = self.authorization_header
            response = api_get(f"{self.host}/admin/me", headers=headers)
            response = response.json()
            org_id = response.get("organization_id")
        except Exception:
            logger.warning("Could not check org_id remotely using default namespace")
        return org_id


class ComponentHandler:

    def __init__(self, context):
        self.context = context
        self.user_handler = UserHandler(context)

    def upload_component(self, spec: Spec, local_path):
        versioned_name = f"{spec.name}-{spec.version}"
        compressed_filename = f"{versioned_name}.{COMPRESSION_TYPE}"
        with Loader("Pushing component to Splight Hub..."):
            try:
                with py7zr.SevenZipFile(compressed_filename, 'w') as archive:
                    archive.writeall(local_path, versioned_name)
                headers = self.user_handler.authorization_header
                data = {
                    'name': spec.name,
                    'version': spec.version,
                    'splight_cli_version': spec.splight_cli_version,
                    'privacy_policy': spec.privacy_policy.value if isinstance(spec.privacy_policy, PrivacyPolicy) else spec.privacy_policy,
                    'tags': spec.tags,
                    'custom_types': json.dumps([x.dict() for x in spec.custom_types]),
                    'input': json.dumps([x.dict() for x in spec.input]),
                    'output': json.dumps([x.dict() for x in spec.output]),
                    'commands': json.dumps([x.dict() for x in spec.commands]),
                    'bindings': json.dumps([x.dict() for x in spec.bindings]),
                    'endpoints': json.dumps([x.dict() for x in spec.endpoints]),
                }
                files = {
                    'file': open(compressed_filename, 'rb'),
                    'readme': open(
                        os.path.join(local_path, README_FILE), 'rb'
                    )
                }
                response = api_post(
                    f"{self.user_handler.host}/hub/upload/",
                    files=files,
                    data=data,
                    headers=headers
                )
                if response.status_code != 201:
                    raise Exception(f"Failed to push component: {response.text}")
            except Exception as e:
                raise e
            finally:
                if os.path.exists(compressed_filename):
                    os.remove(compressed_filename)

    def download_component(self, name, version):
        with Loader("Pulling component from Splight Hub..."):
            headers = self.user_handler.authorization_header
            data = {
                'name': name,
                'version': version,
            }
            response = api_post(
                f"{self.user_handler.host}/hub/download/",
                data=data,
                headers=headers
            )

            if response.status_code != 200:
                if response.status_code == 404:
                    raise Exception("Component not found")
                raise Exception("Failed to pull the component from splight hub")

            versioned_name = f"{name}-{version}"
            compressed_filename = f"{versioned_name}.{COMPRESSION_TYPE}"

            component_path = f"{name}/{version}"
            if os.path.exists(component_path):
                shutil.rmtree(component_path)
            try:
                with open(compressed_filename, "wb") as f:
                    f.write(response.content)

                with py7zr.SevenZipFile(compressed_filename, "r") as z:
                    z.extractall(path='.')
                shutil.move(f"{versioned_name}", component_path)
            finally:
                if os.path.exists(compressed_filename):
                    os.remove(compressed_filename)

    def delete_component(self, name, version):
        with Loader("Deleting component from Splight Hub..."):
            headers = self.user_handler.authorization_header
            response = api_get(
                f"{self.user_handler.host}/hub/mine/component-versions/?name={name}&version={version}",
                headers=headers
            )
            response = response.json()
            for item in response["results"]:
                response = api_delete(
                    f"{self.user_handler.host}/hub/mine/component-versions/{item['id']}/",
                    headers=headers
                )
                if response.status_code != 204:
                    raise Exception(
                        "Failed to delete the component from splight hub"
                    )

    def list_components(self):
        with Loader("Listing components.."):
            headers = self.user_handler.authorization_header
            list_ = []
            page = api_get(
                f"{self.user_handler.host}/hub/mine/components/",
                headers=headers
            )
            page = page.json()
            if page["results"]:
                list_.extend(page["results"])
            while page["next"] is not None:
                page = api_get(page["next"], headers=headers)
                page = page.json()
                if page["results"]:
                    list_.extend(page["results"])
        return list_

    def list_component_versions(self, name):
        with Loader("Listing component versions.."):
            headers = self.user_handler.authorization_header
            list_ = []
            page = api_get(
                f"{self.user_handler.host}/hub/mine/component-versions/?name={name}",
                headers=headers
            )
            page = page.json()
            if page["results"]:
                list_.extend(page["results"])
            while page["next"] is not None:
                page = api_get(page["next"], headers=headers)
                page = page.json()
                if page["results"]:
                    list_.extend(page["results"])
        return list_

    def exists_in_hub(self, name, version):
        headers = self.user_handler.authorization_header
        endpoints = [
            f"{self.user_handler.host}/hub/private/component-versions/?name={name}&version={version}"
            f"{self.user_handler.host}/hub/public/component-versions/?name={name}&version={version}"
        ]
        for endpoint in endpoints:
            response = api_get(endpoint, headers=headers)
            response = response.json()
            if response["count"] > 0:
                break
        return response["count"] > 0

    def create_component(
        self,
        component_name: str,
        component_version: str,
    ) -> Dict:
        endpoint = f"{self.user_handler.host}/"
        headers = self.user_handler.authorization_header
        data = {
            "name": f"CLI: {component_name}",
            "description": "created by Splight CLI",
            "version": component_version,
            "custom_types": [],
            "input": [],
            "output": [],
        }
        response = api_post(endpoint, headers=headers, data=data)
        response.raise_for_status()
        return response.json()

    def get_component_info(
        self,
        component_name: str,
        component_version: str
    ) -> Optional[Dict]:
        endpoint = f"{self.user_handler.host}/hub/all/component-versions/"
        headers = self.user_handler.authorization_header
        params = {
            "name": component_name,
            "version": component_version,
        }
        response = api_get(
            endpoint,
            headers=headers,
            params=params,
        )
        response.raise_for_status()
        component_list = response.json()["results"]
        component = component_list[0] if component_list else None
        return component



import click
from click_default_group import DefaultGroup

from cli.context import Context, pass_context, needs_credentials
from cli.version import __version__


@click.group()
@click.version_option(version=__version__, message="%(version)s")
@click.pass_context
def cli(ctx):
    ctx.obj = Context()


@cli.group()
@pass_context
@needs_credentials
def component(ctx):
    ctx.obj = Context()


@cli.group()
@pass_context
@needs_credentials
def datalake(ctx):
    ctx.obj = Context()


@cli.group()
@pass_context
@needs_credentials
def database(ctx):
    ctx.obj = Context()


@cli.group()
@pass_context
@needs_credentials
def deployment(ctx):
    ctx.obj = Context()


@cli.group()
@pass_context
def workspace(ctx):
    ctx.obj = Context()


@cli.group(cls=DefaultGroup, default="config", default_if_no_args=True)
@pass_context
def configure(ctx: Context, from_json: str = False) -> None:
    ctx.obj = Context()



from cli.constants import *
from cli.datalake.datalake import Datalake
from cli.tests.test_generic import SplightCLITest
from cli.datalake import list
from unittest.mock import patch



class TestList(SplightCLITest):
    def setUp(self):
        super().setUp()
        self.remote_collections = [
            {
                'source': 'default',
                'component': '-'
            },
            {
                'source': 'default1',
                'component': '-'
            }
        ]
        self.configure()

    def test_list(self):
        with patch.object(Datalake, "list", return_value=self.remote_collections):
            result = self.runner.invoke(list, obj=self.context, catch_exceptions=False)
            _stdout = result.output
        self.assertIn("COLLECTION_NAME", _stdout)
        self.assertIn("default", _stdout)
        self.assertIn("default1", _stdout)


import click
import logging
from cli.cli import database as database_cli
from cli.database.database import Database
from cli.utils import *
from cli.context import Context, pass_context


logger = logging.getLogger()


@database_cli.command()
@click.argument('obj_class')
@pass_context
def list(context: Context, obj_class: str) -> None:
    try:
        database = Database(context)
        items = database.list(obj_class)
        Printer.print_dict(items=items, headers=['id', 'name'])

    except Exception as e:
        click.secho(f"Error listing database: {str(e)}", fg="red")
        return


import json
import os
from unittest.mock import patch

import py7zr
import requests

from cli.component.component import Component, ComponentAlreadyExistsException
from cli.component.handler import ComponentHandler
from cli.constants import COMPRESSION_TYPE, README_FILE
from cli.tests.test_generic import SplightCLITest


class TestPush(SplightCLITest):
    def setUp(self):
        super().setUp()
        self.component = Component(self.context)

    @patch.object(ComponentHandler, "upload_component", return_value=None)
    @patch.object(ComponentHandler, "exists_in_hub", return_value=False)
    def test_push(self, exists_in_hub, uploader):
        self.component.push(self.path, force=False)
        exists_in_hub.assert_called_with(self.name, self.version)
        uploader.assert_called_with(self.spec, local_path=self.path)

    @patch.object(ComponentHandler, "upload_component", return_value=None)
    @patch.object(ComponentHandler, "exists_in_hub", return_value=True)
    def test_push_already_exists(self, exists_in_hub, uploader):
        with self.assertRaises(ComponentAlreadyExistsException):
            self.component.push(self.path, force=False)
        exists_in_hub.assert_called_with(self.name, self.version)
        uploader.assert_not_called()

    @patch.object(ComponentHandler, "upload_component", return_value=None)
    @patch.object(ComponentHandler, "exists_in_hub", return_value=True)
    def test_push_forced(self, exists_in_hub, uploader):
        self.component.push(path=self.path, force=True)
        uploader.assert_called_with(self.spec, local_path=self.path)

    @patch.object(ComponentHandler, "exists_in_hub", return_value=False)
    def test_component_upload(self, exists_in_hub):
        self.maxDiff = None
        headers = {
            "Authorization": f"Splight {self.context.workspace.settings.SPLIGHT_ACCESS_ID} {self.context.workspace.settings.SPLIGHT_SECRET_KEY}"
        }
        data = {
            'name': self.spec.name,
            'version': self.spec.version,
            'splight_cli_version': self.spec.splight_cli_version,
            'privacy_policy': self.spec.privacy_policy,
            'tags': self.spec.tags,
            'custom_types': json.dumps([x.dict() for x in self.spec.custom_types]),
            'input': json.dumps([x.dict() for x in self.spec.input]),
            'output': json.dumps([x.dict() for x in self.spec.output]),
            'commands': json.dumps([x.dict() for x in self.spec.commands]),
            'bindings': json.dumps([x.dict() for x in self.spec.bindings]),
            'endpoints': json.dumps([x.dict() for x in self.spec.endpoints]),
        }

        response = requests.Response()
        response.status_code = 201
        with patch.object(requests, "post", return_value=response) as post:
            with patch.object(py7zr.SevenZipFile, "writeall") as writeall:
                self.component.push(self.path, force=False)
                writeall.assert_called_with(
                    self.path, f"{self.name}-{self.version}"
                )
                compressed_filename = (
                    f"{self.name}-{self.version}.{COMPRESSION_TYPE}"
                )
                _, args, kwargs = post.mock_calls[0]
                self.assertEqual(
                    args[0],
                    f"{self.context.workspace.settings.SPLIGHT_PLATFORM_API_HOST}/hub/upload/",
                )
                self.assertEqual(
                    kwargs["files"]["file"].name, compressed_filename
                )
                self.assertEqual(
                    kwargs["files"]["readme"].name,
                    os.path.join(self.path, README_FILE),
                )
                self.assertEqual(
                    kwargs["headers"], {**kwargs["headers"], **headers}
                )
                self.assertEqual(
                    kwargs["data"], data
                )



import uuid
import os
from typing import List, Optional, Dict
from jinja2 import Template
from splight_lib.execution import Thread
from splight_lib import logging
from cli.version import __version__
from cli.constants import (
    COMPONENT_FILE,
)
from cli.utils import get_template
from cli.component.handler import ComponentHandler
from cli.component.spec import Spec
from cli.component.loaders import SpecLoader, ComponentLoader, InitLoader
from cli.component.exceptions import ComponentAlreadyExistsException, ComponentDirectoryAlreadyExists


logger = logging.getLogger()


class Component:
    name = None
    version = None

    def __init__(self, context):
        self.context = context

    def list(self):
        handler = ComponentHandler(self.context)
        return handler.list_components()

    def versions(self, name):
        handler = ComponentHandler(self.context)
        return handler.list_component_versions(name)

    def create(self, name, version):
        Spec.verify({
            "name": name,
            "version": version,
            "splight_cli_version": __version__,
            "custom_types": [],
            "input": [],
            "output": [],
            "endpoints": [],
        })

        component_path = os.path.join(f"{name}", f"{version}")
        os.makedirs(component_path)

        for file_name in ComponentLoader.REQUIRED_FILES:
            template_name = file_name
            file_path = os.path.join(component_path, file_name)
            component_id = str(uuid.uuid4())
            if file_name == COMPONENT_FILE:
                template_name = "component.py"
            template: Template = get_template(template_name)
            file = template.render(
                component_name=name,
                version=version,
                component_id=component_id,
                splight_cli_version=__version__
            )
            with open(file_path, "w+") as f:
                f.write(file)

    def pull(self, name, version):
        versioned_name = f"{name}-{version}"
        component_path = os.path.join(versioned_name)

        if os.path.exists(component_path):
            raise ComponentDirectoryAlreadyExists(versioned_name)

        handler = ComponentHandler(self.context)
        handler.download_component(name, version)

    def delete(self, name, version):
        handler = ComponentHandler(self.context)
        handler.delete_component(name, version)

    def push(self, path: str, force = False):
        # Load json and validate Spec structure 
        loader = SpecLoader(path=path)
        spec = loader.load(prompt_input=False)

        # TODO simplify this
        handler = ComponentHandler(self.context)
        if not force and handler.exists_in_hub(spec.name, spec.version):
            raise ComponentAlreadyExistsException(f'{spec.name}-{spec.version}')
        handler.upload_component(spec, local_path=path)

    def run(self, path: str, input_parameters: Optional[List[Dict]] = None):
        # Load py module and validate Splight Component structure 
        loader = ComponentLoader(path=path)
        component_class = loader.load()
        # Load json and validate Spec structure 
        loader = SpecLoader(path=path)
        run_spec = loader.load(input_parameters=input_parameters)
        component = component_class(
            run_spec=run_spec.dict(),
            initial_setup=self.context.workspace.settings.dict()
        )
        component.execution_client.start(Thread(target=component.start))

    def install_requirements(self, path: str):
        loader = InitLoader(path=path)
        loader.load()



import os
from unittest.mock import patch


from cli.component.component import Component
from cli.component.handler import ComponentHandler
from cli.tests.test_generic import SplightCLITest


class TestDelete(SplightCLITest):
    def setUp(self):
        super().setUp()
        self.original_component_path = self.path
        self.path = os.path.dirname(os.path.abspath(__file__))
        self.extracted_component_path = os.path.join(
            self.path, f"{self.name}-{self.version}"
        )
        self.component = Component(self.context)

    def test_component_delete(self):
        with patch.object(
            ComponentHandler, "delete_component"
        ) as call:
            self.component.delete(self.name, self.version)
            call.assert_called_with(
                self.name, self.version
            )


import splight_models
from splight_lib import logging
from cli.utils import *
from cli.constants import DEFAULT_NAMESPACE


logger = logging.getLogger()


class Database:

    def __init__(self, context):
        self.context = context
        self.namespace = DEFAULT_NAMESPACE

    @property
    def client(self):
        return self.context.framework.setup.DATABASE_CLIENT(self.namespace)

    def list(self, obj_class):
        obj_class = getattr(splight_models, obj_class.title())
        return [
            i.dict()
            for i in self.client.get(obj_class)
        ]


import sys
import json
from typing import Any

import click

from cli.cli import configure as cli_config
from cli.context import pass_context
from cli.context import Context
from cli.settings import SplightCLISettings, CONFIG_VARS


@cli_config.command()
@click.option(
    "-j",
    "--from-json",
    help="Configuration by json instaed of prompt."
)
@pass_context
def config(ctx: Context, from_json: str = False) -> None:
    try:
        if from_json:
            from_json = json.loads(from_json)
            new_settings = SplightCLISettings.parse_obj(from_json)
        else:
            new_settings_data = {}
            for config_var in CONFIG_VARS:
                default = getattr(ctx.workspace.settings, config_var, None)
                value = click.prompt(
                    click.style(config_var, fg='yellow'),
                    type=str,
                    default=default,
                    show_default=True
                )
                new_settings_data.update({config_var: value})
            new_settings = SplightCLISettings.parse_obj(new_settings_data)
        ctx.workspace.update_workspace(new_settings)
        click.secho("Configuration saved successfully", fg="green")
    except Exception as e:
        click.secho(f"Error configuring Splight CLI: {str(e)}", fg="red")
        sys.exit(1)


@cli_config.command(name="get")
@click.argument(
    "var_name"
)
@pass_context
def get_variable(ctx: Context, var_name: str) -> None:
    """Prints the value of the requested variable.

    Parameters
    ----------
    ctx: Context
        The current context
    var_name:
        The variable name to get the value
    """
    variable_name = var_name.upper()
    settings = ctx.workspace.settings

    if hasattr(settings, variable_name):
        value = getattr(settings, variable_name)
        click.secho(value)


@cli_config.command(name="set")
@click.argument(
    "var_name"
)
@click.argument(
    "value"
)
@pass_context
def set_variable(ctx: Context, var_name: str, value: Any) -> None:
    """Sets a value of a variable in the current workspace.

    Parameters
    ----------
    ctx: Context
        The current context
    var_name: str
        The variable's name to be updated
    value: Any
        The new value
    """
    variable_name = var_name.upper()
    settings = ctx.workspace.settings

    if hasattr(settings, variable_name):
        setattr(settings, variable_name, value)
        ctx.workspace.update_workspace(settings)



from pathlib import Path
from typing import Dict, List
from cli.settings import SplightCLISettings
from cli.utils.yaml import get_yaml_from_file, save_yaml_to_file
from cli.constants import CONFIG_FILE, DEFAULT_WORKSPACE, DEFAULT_WORKSPACE_NAME, DEFAULT_WORKSPACES


class WorkspaceDeleteError(Exception):
    def __init__(self, workspace: str):
        self._msg = (
            f"Workspace '{workspace}' is your active workspace\n\n"
            "You cannot delete the currently active workspace"
        )

    def __str__(self) -> str:
        return self._msg


class WorkspaceManager:
    def __init__(self):
        self.config_file = CONFIG_FILE
        Path(self.config_file).touch()
        self._settings = self.__load_config()
        self._workspaces: List[Dict] = self._settings.get('workspaces')
        self._current_workspace: str = self._settings.get('current_workspace')
        self._current_settings = SplightCLISettings.parse_obj(self._workspaces[self._current_workspace])

    def __load_config(self):
        config = get_yaml_from_file(self.config_file)
        # Default values
        config['workspaces'] = config.get('workspaces', DEFAULT_WORKSPACES)
        workspace_name = config.get(
            "current_workspace", DEFAULT_WORKSPACE_NAME
        )
        if workspace_name not in config["workspaces"]:
            workspace_name = (
                DEFAULT_WORKSPACE_NAME
                if DEFAULT_WORKSPACE_NAME in config["workspaces"]
                else list(config["workspaces"].keys())[0]
            )
        config["current_workspace"] = workspace_name
        save_yaml_to_file(config, self.config_file)
        return config

    @property
    def settings(self):
        return self._current_settings

    @property
    def current_workspace(self):
        return self._current_workspace

    def update_workspace(self, new_settings):
        self._workspaces[self._current_workspace] = new_settings.dict()
        save_yaml_to_file(self._settings, self.config_file)

    def select_workspace(self, value):
        self._settings['current_workspace'] = value
        save_yaml_to_file(self._settings, self.config_file)

    def list_workspaces(self):
        workspaces = self._settings.get('workspaces').keys()
        return [f"{key}*" if key == self._current_workspace else f"{key}" for key in workspaces]

    def delete_workspace(self, workspace_name):
        if workspace_name not in self._settings['workspaces']:
            raise Exception('Not a valid namespace name')

        if workspace_name == self._current_workspace:
            raise WorkspaceDeleteError(workspace_name)
        del self._settings['workspaces'][workspace_name]
        save_yaml_to_file(self._settings, self.config_file)

    def create_workspace(self, workspace_name):
        if workspace_name in self._settings['workspaces']:
            raise Exception('Name already exists')
        self._settings['workspaces'].update({
            workspace_name: DEFAULT_WORKSPACE
        })
        self._settings['current_workspace'] = workspace_name
        self._current_workspace = workspace_name
        save_yaml_to_file(self._settings, self.config_file)



import click
from functools import wraps
from cli.context.workspace import WorkspaceManager
from cli.context.framework import FrameworkManager

class Context:
    def __init__(self):
        self.__workspace = WorkspaceManager()
        self.__framework = FrameworkManager()

    @property
    def framework(self):
        self.__framework.configure(self.workspace.settings.dict())
        return self.__framework

    @property
    def workspace(self):
        return self.__workspace

    def __repr__(self):
        return f"<Context {self.__workspace}>"


pass_context = click.make_pass_decorator(Context)


def needs_credentials(f):
    @wraps(f)
    def wrapper(ctx, *args, **kwargs):
        access_id = ctx.workspace.settings.SPLIGHT_ACCESS_ID
        secret_key = ctx.workspace.settings.SPLIGHT_SECRET_KEY
        if access_id is None or secret_key is None:
            click.secho(
                "Please set your Splight credentials. Use \"splightcli configure\"",
                fg="red"
            )
            exit(1)
        return f(ctx, *args, **kwargs)
    return wrapper



import pandas as pd
import splight_models as models
from datetime import datetime
from cli.utils import *
from cli.constants import DEFAULT_NAMESPACE


class Datalake:

    def __init__(self, context):
        self.context = context
        self.namespace = DEFAULT_NAMESPACE

    @property
    def client(self):
        return self.context.framework.setup.DATALAKE_CLIENT(self.namespace)

    @property
    def sample_dataframe(self):
        return pd.read_csv(f"{BASE_DIR}/cli/datalake/dump_example.csv")

    def list(self):
        return self.client.list_collection_names()

    def dump(self, collection, path, filter, example):
        if os.path.exists(path):
            raise Exception(f"File {path} already exists")
        if os.path.isdir(path):
            path = os.path.join(path, 'splight_dump.csv')
        elif not path.endswith(".csv"):
            raise Exception("Only CSV files are supported")

        if example:
            dataframe = self.sample_dataframe
        else:
            filters = {
                f.split('=')[0]: f.split('=')[1] for f in filter
            }

            dataframe = self.client.get_dataframe(
                resource_type=models.Variable,
                collection=collection,
                **self._get_filters()
            )
        dataframe.to_csv(path, index=False)
        click.secho(f"Succesfully dumpped {collection} in {path}", fg="green")

    def load(self, collection, path, example):
        if not os.path.isfile(path):
            raise Exception("File not found")
        if not path.endswith(".csv"):
            raise Exception("Only CSV files are supported")

        if example:
            dataframe = self.sample_dataframe
        else:
            dataframe = pd.read_csv(path)
        self.client.save_dataframe(dataframe, collection=collection)
        click.secho(f"Succesfully loaded {path} in {collection}", fg="green")

    @staticmethod
    def _to_list(key: str, elem: str):
        if ',' not in elem and '__in' not in key:
            raise ValueError
        return list(elem.split(","))

    @staticmethod
    def _to_int(key: str, elem: str):
        if elem[0] == '-':
            can = elem[1:].isdigit()
        else:
            can = elem.isdigit()
        if not can:
            raise ValueError
        return int(elem)

    @staticmethod
    def _to_float(key: str, elem: str):
        try:
            return float(elem)
        except Exception:
            raise ValueError

    @staticmethod
    def _to_bool(key: str, elem: str):
        if elem == "True":
            return True
        elif elem == "False":
            return False
        else:
            raise ValueError

    @staticmethod
    def _to_date(key: str, elem: str):
        return datetime.strptime(elem, "%Y-%m-%dT%H:%M:%S%z")

    def _get_filters(self, filters):
        to_cast = [
            self._to_list,
            self._to_int,
            self._to_float,
            self._to_bool,
            self._to_date
        ]
        parsed_filters = {}
        for key, value in filters.items():
            if key == 'limit_':
                parsed_filters[key] = value
                continue
            for cast in to_cast:
                try:
                    value = cast(key, value)
                    break
                except ValueError:
                    pass

            parsed_filters[key] = value
        return parsed_filters
